#-------------------------------------------------------------------------------
# Name:        LinkedList.py
# Purpose:     A LinkedList class for visualizing the object connections
#              of a dynamic data structure in the uml window.
#
# Author:      Gerhard Röhner
#
# Created:     31.01.2022
# Copyright:   (c)  2022
# Licence:     Freeware
# Reference:   https://realpython.com/linked-lists-python/
#-------------------------------------------------------------------------------

class Node:
    def __init__(self, data):
        self.data = data
        self.next = None

class LinkedList:
    def __init__(self, nodes=None):
        self.head = None
        if nodes is not None:
            node = Node(data=nodes.pop(0))
            self.head = node
            for elem in nodes:
                node.next = Node(data=elem)
                node = node.next

    def insert_first(self, node):
        node.next = self.head
        self.head = node

    def insert_last(self, node):
        if self.head is None:
            self.head = node
        else:
            cursor = self.head
            while cursor.next is not None:
                cursor = cursor.next
            cursor.next = node

    def insert_after(self, data, new_node):
        cursor = self.head
        while cursor is not None and cursor.data != data:
            cursor = cursor.next
        if cursor is not None:
            new_node.next = cursor.next
            cursor.next = new_node
        else:
            raise Exception("Node with data '%s' not found" % data)

    def insert_before(self, data, new_node):
        if self.head.data == data:
            self.insert_first(new_node)
            return

        prev_node = self.head
        cursor = prev_node.next
        while cursor is not None and cursor.data != data:
            prev_node = cursor
            cursor = cursor.next
        if cursor is not None:
            new_node.next = cursor
            prev_node.next = new_node
        else:
            raise Exception("Node with data '%s' not found" % data)

    def remove(self, data):
        if self.head.data == data:
            self.head = self.head.next
        else:
            prev_node = self.head
            cursor = prev_node.next
            while cursor is not None and cursor.data != data:
                prev_node = cursor
                cursor = cursor.next
            if cursor is not None:
                prev_node.next = cursor.next



